# -*- coding: utf-8 -*-
"""
Copyright (C) 2020-2024 LiteyukiStudio. All Rights Reserved 

@Time    : 2024/8/25 下午9:45
@Author  : snowykami
@Email   : snowykami@outlook.com
@File    : const.py
@Software: PyCharm
"""
import math


PI = math.pi
E = math.e
GOLDEN_RATIO = (1 + math.sqrt(5)) / 2
GAMMA = 0.57721566490153286060651209008240243104215933593992
EPSILON = 0.0001
"""ε"""
APPROX = 0.001
"""约等于误差"""
